// $Id: SubjectVariable.java,v 1.6 2004/02/27 16:30:00 m31 Exp $
//
// This file is part of the program suite "Principia Mathematica II"
// which is a working prototype for the main project:
// "Hilbert II" - http://www.qedeq.org
//
// Copyright 2001-2004  Michael Meyling <mime@qedeq.org>.
//
// "Principia Mathematica II" is free software; you can redistribute
// it and/or modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.AbstractArgumentList;
import com.meyling.principia.argument.Argument;
import com.meyling.principia.argument.ArgumentException;
import com.meyling.principia.argument.Counter;
import com.meyling.principia.argument.CounterMarker;


/**
 * Subject variable, represents a subject of the universe of discourse.
 *
 * @version $Revision: 1.6 $
 * @author  Michael Meyling
 */
public class SubjectVariable extends AbstractArgumentList
        implements SubjectVariableMarker {


    /** the number of the subject variable */
    private CounterMarker number;


    /**
     * Constructs certain subject variable.
     *
     * @param   arguments  first argument is number of subject variable
     * @throws  ArgumentException    if there is not exactly one
     *          argument that is an instance of {@link CounterMarker}
     */
    public SubjectVariable(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 1) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                BasicCreator.getName(SubjectVariable.class)
                + BasicConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof CounterMarker)) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                BasicCreator.getName(SubjectVariable.class)
                + BasicConstants.FIRST_ARGUMENT_TYPE
                + BasicConstants.COUNTER,
                arguments[0]);
        }
        this.number = (CounterMarker) arguments[0];
    }


    /**
     * Constructs certain subject variable.
     *
     * @param   number  number of subject variable
     * @throws  ArgumentException    if Counter couldn't be created
     */
    public SubjectVariable(final int number)
            throws ArgumentException {
        this(new Argument[] { new Counter(number) } );
    }


    /**
     * Get the number of the subject variable.
     *
     * @return  number of subject variable
     */
    public final int getNumber() {
        return number.getNumber();
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new SubjectVariable(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "x" + number.toString();
    }


}
